Method: OpenKey::KeyDb#get_entry

Defined in:
lib/keytools/key.db.rb

#get_entry(dictionary_name, key_name) ⇒ Object

Get the entry with the key name in a dictionary that is itself inside another dictionary (named in the first parameter) which thankfully is at the root of this database.

Only call this method if #has_entry? returns true for the same dictionary and key name parameters.

Parameters:

  • dictionary_name (String)

    get the entry inside a dictionary which is itself inside a dictionary (with this dictionary name) which is itself at the root of this database.

  • key_name (String)

    get the value part of the key value pair that is inside a dictionary (with the above dictionary name) which is itself at the root of this database.



194
195
196
197
198
# File 'lib/keytools/key.db.rb', line 194

def get_entry( dictionary_name, key_name )

  return self[ dictionary_name ][ key_name ]

end